@@ -30,7 +30,7 @@ from PyQt4.QtGui import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, \
     QLabel, QComboBox, QLineEdit, QPushButton, QFormLayout, \
     QProgressBar, QRadioButton, QSpacerItem, QSizePolicy
 
-from kde import KMessageBox, KDialogButtonBox, KUser, KIcon
+from kde import Sorry, Information, QuestionYesNo, KDialogButtonBox, KUser, KIcon
 
 from util import m18n, m18nc, logWarning, logException, socketName, english, \
     appdataDir, logInfo, logDebug, removeIfExists, which
@@ -41,7 +41,7 @@ from common import InternalParameters, PREF, Debug
 from game import Players
 from query import Transaction, Query
 from board import Board
-from client import Client
+from client import Client, ClientTable
 from statesaver import StateSaver
 from meld import Meld
 from tables import TableList
@@ -49,7 +49,7 @@ from sound import Voice
 import intelligence
 import altint
 
-from guiutil import ListComboBox
+from guiutil import ListComboBox, DialogIgnoringEscape
 from rule import Ruleset
 
 class LoginAborted(Exception):
@@ -269,10 +269,10 @@ class AddUserDialog(QDialog):
             self.edPassword.setText(password)
         return property(**locals())
 
-class SelectChow(QDialog):
+class SelectChow(DialogIgnoringEscape):
     """asks which of the possible chows is wanted"""
     def __init__(self, chows, propose):
-        QDialog.__init__(self)
+        DialogIgnoringEscape.__init__(self)
         self.setWindowTitle('Kajongg')
         self.chows = chows
         self.selectedChow = None
@@ -302,17 +302,10 @@ class SelectChow(QDialog):
         else:
             event.ignore()
 
-    def keyPressEvent(self, event):
-        """catch and ignore the Escape key"""
-        if event.key() == Qt.Key_Escape:
-            event.ignore()
-        else:
-            QDialog.keyPressEvent(self, event)
-
-class SelectKong(QDialog):
+class SelectKong(DialogIgnoringEscape):
     """asks which of the possible kongs is wanted"""
     def __init__(self, kongs):
-        QDialog.__init__(self)
+        DialogIgnoringEscape.__init__(self)
         self.setWindowTitle('Kajongg')
         self.kongs = kongs
         self.selectedKong = None
@@ -339,13 +332,6 @@ class SelectKong(QDialog):
         else:
             event.ignore()
 
-    def keyPressEvent(self, event):
-        """catch and ignore the Escape key"""
-        if event.key() == Qt.Key_Escape:
-            event.ignore()
-        else:
-            QDialog.keyPressEvent(self, event)
-
 class DlgButton(QPushButton):
     """special button for ClientDialog"""
     def __init__(self, message, parent):
@@ -465,6 +451,22 @@ class ClientDialog(QDialog):
         """a list of all messages returned by the declared buttons"""
         return list(x.message for x in self.buttons)
 
+    def proposeAction(self):
+        """either intelligently or first button by default. May also
+        focus a proposed tile depending on the action."""
+        result = self.buttons[0]
+        game = self.client.game
+        if game.autoPlay or PREF.propose:
+            answer, parameter = self.client.intelligence.selectAnswer(
+                self.messages())
+            result = [x for x in self.buttons if x.message == answer][0]
+            result.setFocus()
+            if answer in [Message.Discard, Message.OriginalCall]:
+                for tile in game.myself.handBoard.tiles:
+                    if tile.element == parameter:
+                        game.myself.handBoard.focusTile = tile
+        return result
+
     def askHuman(self, move, answers, deferred):
         """make buttons specified by answers visible. The first answer is default.
         The default button only appears with blue border when this dialog has
@@ -478,17 +480,7 @@ class ClientDialog(QDialog):
         self.checkTiles()
         game = self.client.game
         myTurn = game.activePlayer == game.myself
-        prefButton = self.buttons[0]
-        if game.autoPlay or PREF.propose:
-            answer, parameter = self.client.intelligence.selectAnswer(
-                self.messages())
-            prefButton = [x for x in self.buttons if x.message == answer][0]
-            prefButton.setFocus()
-            if answer in [Message.Discard, Message.OriginalCall]:
-                for tile in game.myself.handBoard.tiles:
-                    if tile.element == parameter:
-                        game.myself.handBoard.focusTile = tile
-
+        prefButton = self.proposeAction()
         if game.autoPlay:
             self.selectButton(prefButton)
             return
@@ -565,8 +557,7 @@ class ClientDialog(QDialog):
             else:
                 answer = button.message
             if not self.client.sayable[answer]:
-                message = m18n('You cannot say %1', answer.i18nName)
-                KMessageBox.sorry(None, message)
+                Sorry(m18n('You cannot say %1', answer.i18nName))
                 return
             self.deferred.callback(answer)
         self.hide()
@@ -577,35 +568,6 @@ class ClientDialog(QDialog):
         if not self.client.game.autoPlay:
             self.selectButton(self.sender())
 
-class ReadyHandQuestion(QDialog):
-    """ask user if he is ready for the hand"""
-    def __init__(self, deferred, parent=None):
-        QDialog.__init__(self, parent)
-        self.setAttribute(Qt.WA_DeleteOnClose)
-        self.deferred = deferred
-        layout = QVBoxLayout(self)
-        buttonBox = QDialogButtonBox()
-        layout.addWidget(buttonBox)
-        self.okButton = buttonBox.addButton(m18n("&Ready for next hand?"),
-          QDialogButtonBox.AcceptRole)
-        self.okButton.clicked.connect(self.accept)
-        self.setWindowTitle('Kajongg')
-        buttonBox.accepted.connect(self.accept)
-        buttonBox.rejected.connect(self.accept)
-
-    def accept(self):
-        """player is ready"""
-        if self.isVisible():
-            self.deferred.callback(None)
-            self.hide()
-
-    def keyPressEvent(self, event):
-        """catch and ignore the Escape key"""
-        if event.key() == Qt.Key_Escape:
-            event.ignore()
-        else:
-            QDialog.keyPressEvent(self, event)
-
 class AlreadyConnected(Exception):
     """we already have a connection to the server"""
     def __init__(self, url):
@@ -627,14 +589,12 @@ class HumanClient(Client):
         self.tableList = None
         self.connector = None
         self.table = None
-        self.readyHandQuestion = None
         self.loginDialog = LoginDialog()
         if InternalParameters.autoPlay:
             self.loginDialog.accept()
         else:
             if not self.loginDialog.exec_():
                 InternalParameters.field.startingGame = False
-                InternalParameters.field.updateGUI()
                 raise LoginAborted
         self.useSocket = self.loginDialog.host == Query.localServerName
         self.assertConnectivity()
@@ -653,6 +613,15 @@ class HumanClient(Client):
                 client.tableList.activateWindow()
                 raise AlreadyConnected(self.__url)
 
+    def pingLater(self, dummyResult):
+        """ping the server every 5 seconds"""
+        InternalParameters.reactor.callLater(5, self.ping)
+
+    def ping(self):
+        """regularly check if server is still there"""
+        if self.perspective:
+            self.callServer('ping').addCallback(self.pingLater).addErrback(self.remote_serverDisconnects)
+
     @staticmethod
     def __findAI(modules, aiName):
         """list of all alternative AIs defined in altint.py"""
@@ -750,10 +719,10 @@ class HumanClient(Client):
             # riverbank.computing says module dbus is deprecated
             # for Python 3. And Ubuntu has no package with
             # PyQt4.QtDBus. So we use good old subprocess.
-            answer = str(subprocess.check_output(['qdbus',
+            answer = subprocess.Popen(['qdbus',
                 'org.kde.kded',
                 '/modules/networkstatus',
-                'org.kde.Solid.Networking.status'])).strip()
+                'org.kde.Solid.Networking.status'], stdout=subprocess.PIPE).communicate()[0].strip()
             if answer != '4':
                 raise NetworkOffline(answer)
 
@@ -798,6 +767,19 @@ class HumanClient(Client):
 
     def remote_replaceTable(self, table):
         """update table list"""
+        newClientTable = ClientTable.fromList(self, table)
+        oldTable = self.tableById(newClientTable.tableid)
+        if oldTable:
+            # this happens if a game has more than one human player and
+            # one of them answers "no" to "are you ready to begin". In
+            # that case, the other clients need this code. Otherwise they
+            # would start the game anyway, and the user would have to abort it
+            if newClientTable.isOnline(self.username):
+                for name in newClientTable.playerNames:
+                    if name != self.username:
+                        if oldTable.isOnline(name) and not newClientTable.isOnline(name):
+                            # TODO: has left table is better, but string freeze
+                            Sorry(m18n('Player %1 has logged out', name), self.logout)
         Client.remote_replaceTable(self, table)
         self.tableList.loadTables(self.tables)
 
@@ -817,44 +799,47 @@ class HumanClient(Client):
                 # TODO: chatting on a table with suspended game does
                 # not yet work because such a table has no tableid. Maybe it should.
                 return
-        if not chatLine.isStatusMessage:
-            ChatWindow.createFor(table)
+        if not chatLine.isStatusMessage and not table.chatWindow:
+            ChatWindow(table)
         if table.chatWindow:
             table.chatWindow.receiveLine(chatLine)
 
     def readyForGameStart(self, tableid, gameid, wantedGame, playerNames, shouldSave=True):
         """playerNames are in wind order ESWN"""
+        def answered(result):
+            """callback, called after the client player said yes or no"""
+            if self.perspective and result:
+                # still connected and yes, we are
+                return Client.readyForGameStart(self, tableid, gameid, wantedGame, playerNames, shouldSave)
+            else:
+                return Message.NO
         self.tableList.hide()
         if sum(not x[1].startswith('Robot ') for x in playerNames) == 1:
             # we play against 3 robots and we already told the server to start: no need to ask again
-            wantStart = True
-        else:
-            msg = m18n("The game can begin. Are you ready to play now?\n" \
-                "If you answer with NO, you will be removed from the table.")
-            wantStart = KMessageBox.questionYesNo (None, msg) == KMessageBox.Yes
-        if wantStart:
-            return Client.readyForGameStart(self, tableid, gameid, wantedGame, playerNames, shouldSave=shouldSave)
-        else:
-            return Message.NO
+            return Client.readyForGameStart(self, tableid, gameid, wantedGame, playerNames, shouldSave)
+        msg = m18n("The game can begin. Are you ready to play now?\n" \
+            "If you answer with NO, you will be removed from the table.")
+        return QuestionYesNo(msg, answered)
 
     def readyForHandStart(self, playerNames, rotateWinds):
         """playerNames are in wind order ESWN. Never called for first hand."""
+        def answered(dummy=None):
+            """called after the client player said yes, I am ready"""
+            if self.perspective:
+                # still connected?
+                return Client.readyForHandStart(self, playerNames, rotateWinds)
+        if not self.perspective:
+            # disconnected meanwhile
+            return
         if InternalParameters.field:
             # update the balances in the status bar:
             InternalParameters.field.updateGUI()
         assert not self.game.isFirstHand()
         if self.game.autoPlay:
-            self.clientReadyForHandStart(None, playerNames, rotateWinds)
-            return
-        deferred = Deferred()
-        deferred.addCallback(self.clientReadyForHandStart, playerNames, rotateWinds)
-        self.readyHandQuestion = ReadyHandQuestion(deferred, InternalParameters.field)
-        self.readyHandQuestion.show()
-        return deferred
-
-    def clientReadyForHandStart(self, dummy, playerNames, rotateWinds):
-        """callback, called after the client player said yes, I am ready"""
-        return Client.readyForHandStart(self, playerNames, rotateWinds)
+            return answered()
+        else:
+            # string freeze, but we do not need the & anymore...
+            return Information(m18n("&Ready for next hand?").replace('&', ''), answered)
 
     def ask(self, move, answers):
         """server sends move. We ask the user. answers is a list with possible answers,
@@ -964,12 +949,24 @@ class HumanClient(Client):
                 else:
                     self.game.close().addCallback(Client.quitProgram)
 
-    def remote_serverDisconnects(self):
-        """the kajongg server ends our connection. We remove ourself
-        fromt the list of clients, so we might disappear anytime"""
+    def remote_serverDisconnects(self, dummyResult=None):
+        """we logged out or or lost connection to the server.
+        Remove visual traces depending on that connection."""
         self.perspective = None
+        if self.tableList:
+            model = self.tableList.view.model()
+            if model:
+                for table in model.tables:
+                    if table.chatWindow:
+                        table.chatWindow.hide()
+                        table.chatWindow = None
+            self.tableList.hide()
+            self.tableList = None
         if self in self.clients:
             self.clients.remove(self)
+        field = InternalParameters.field
+        if field and field.game == self.game:
+            field.hideGame()
 
     def loginCommand(self, username):
         """send a login command to server. That might be a normal login
@@ -1022,27 +1019,30 @@ class HumanClient(Client):
 
     def _loginFailed(self, failure):
         """login failed"""
+        def answered(result):
+            """user finally answered our question"""
+            if result:
+                return self.adduser(url, name, passwd)
+            else:
+                InternalParameters.field.startingGame = False
         message = failure.getErrorMessage()
         dlg = self.loginDialog
         if 'Wrong username' in message:
             url, name, passwd = dlg.url, dlg.username, dlg.password
             host = url.split(':')[0]
-            msg = m18nc('USER is not known on SERVER',
-                '%1 is not known on %2, do you want to open an account?', name, host)
-            if url == Query.localServerName \
-            or KMessageBox.questionYesNo (None, msg) == KMessageBox.Yes:
-                return self.adduser(url, name, passwd)
+            if url == Query.localServerName:
+                return self.answered(True)
+            else:
+                msg = m18nc('USER is not known on SERVER',
+                    '%1 is not known on %2, do you want to open an account?', name, host)
+                return QuestionYesNo(msg, answered)
         else:
             self._loginReallyFailed(failure)
 
     def _loginReallyFailed(self, failure):
         """login failed, not fixable by adding missing user"""
-        message = self._prettifyErrorMessage(failure)
-        InternalParameters.reactor.setEnabled(False)
-        try:
-            logWarning(message)
-        finally:
-            InternalParameters.reactor.setEnabled(True)
+        InternalParameters.field.startingGame = False
+        logWarning(self._prettifyErrorMessage(failure))
 
     def adduserOK(self, dummyFailure):
         """adduser succeeded"""
@@ -1069,11 +1069,12 @@ class HumanClient(Client):
         maxGameId = Query('select max(id) from game').records[0][0]
         maxGameId = int(maxGameId) if maxGameId else 0
         self.callServer('setClientProperties',
-            str(Query.dbhandle.databaseName()),
+            InternalParameters.dbIdent,
             voiceId, maxGameId, InternalParameters.version). \
                 addErrback(self.versionError). \
                 addCallback(self.callServer, 'sendTables'). \
                 addCallback(self.tableList.gotTables)
+        self.ping()
 
     @staticmethod
     def versionError(err):
@@ -1137,19 +1138,11 @@ class HumanClient(Client):
             return self.__url
         return property(**locals())
 
-    def logout(self):
+    def logout(self, dummyResult=None):
         """clean visual traces and logout from server"""
         result = None
         if self.perspective:
             result = self.callServer('logout')
-        field = InternalParameters.field
-        if field:
-            field.hideGame()
-        if self.readyHandQuestion:
-            self.readyHandQuestion.hide()
-        if self.table and self.table.chatWindow:
-            self.table.chatWindow.hide()
-            self.table.chatWindow = None
         return result or succeed(None)
 
     def callServer(self, *args):
@@ -1165,11 +1158,11 @@ class HumanClient(Client):
                         logDebug('callServer(%s)' % repr(args))
                 return self.perspective.callRemote(*args)
             except pb.DeadReferenceError:
-                self.perspective = None
-                self.clients.remove(self)
                 logWarning(m18n('The connection to the server %1 broke, please try again later.',
                                   self.url))
+                self.remote_serverDisconnects()
+                return succeed(None)
 
     def sendChat(self, chatLine):
         """send chat message to server"""
-        self.callServer('chat', chatLine.serialize())
+        return self.callServer('chat', chatLine.serialize())
