@@ -251,7 +251,7 @@ class ScoreModel(TreeModel):
                 list([game.gameid])).records
         # pylint: disable=W0142
         # pylint * magic
-        data = list(tuple([m18nc('kajongg', player.name), [HandResult(*x[1:]) for x in records \
+        data = list(tuple([player.localName, [HandResult(*x[1:]) for x in records \
                 if x[0] == player.nameid]]) for player in game.players)
         self.__findMinMaxChartPoints(data)
         parent = QModelIndex()
@@ -550,18 +550,21 @@ class ExplainView(QListView):
             lines.append(m18n('Ruleset: %1', i18nName))
             lines.append('')
             for player in self.game.players:
-                iName = m18nc('kajongg', player.name)
                 pLines = []
-                if player.handContent and player.handContent.tileNames:
-                    total = player.handContent.total()
-                    pLines = ['%s: %s' % (iName, total)]
-                    for line in player.handContent.explain():
-                        pLines.append('- ' + line)
+                if player.hand and player.hand.tileNames:
+                    total = player.newHandContent.total()
+                    if total:
+                        pLines = ['%s: %s' % (player.localName, total)]
+                        for line in player.newHandContent.explain():
+                            pLines.append('- ' + line)
                 elif player.handTotal:
-                    pLines.append(m18n('Manual score for %1: %2 points', iName, player.handTotal))
-                pLines.append('')
+                    pLines.append(m18n('Manual score for %1: %2 points', player.localName, player.handTotal))
+                if pLines:
+                    pLines.append('')
                 lines.extend(pLines)
-        self.model.setStringList(lines)
+        if 'xxx'.join(lines) != 'xxx'.join(unicode(x) for x in self.model.stringList()):
+            # QStringListModel does not optimize identical lists away, so we do
+            self.model.setStringList(lines)
 
 class PenaltyBox(QSpinBox):
     """with its own validator, we only accept multiples of parties"""
@@ -842,10 +845,10 @@ class ScoringDialog(QWidget):
                         del child
                 if game:
                     self.spValues[idx].setRange(0, game.ruleset.limit or 99999)
-                    self.nameLabels[idx].setText(m18nc('kajongg', player.name))
+                    self.nameLabels[idx].setText(player.localName)
                     self.windLabels[idx].wind = player.wind
                     self.windLabels[idx].roundsFinished = game.roundsFinished
-                    self.detailTabs.setTabText(idx, m18nc('kajongg', player.name))
+                    self.detailTabs.setTabText(idx, player.localName)
                     player.manualRuleBoxes = [RuleBox(x) for x in game.ruleset.allRules if x.hasSelectable]
                     for ruleBox in player.manualRuleBoxes:
                         self.detailsLayout[idx].addWidget(ruleBox)
@@ -877,8 +880,7 @@ class ScoringDialog(QWidget):
                 for box in winner.manualRuleBoxes:
                     if box.isChecked():
                         box.setChecked(False)
-                        hand = winner.computeHand()
-                        if hand.manualRuleMayApply(box.rule):
+                        if winner.hand.manualRuleMayApply(box.rule):
                             box.setChecked(True)
         self.prevLastTile = newLastTile
         self.fillLastMeldCombo()
@@ -952,7 +954,7 @@ class ScoringDialog(QWidget):
                 self.spValues[idx].setValue(0)
                 self.wonBoxes[idx].setChecked(False)
                 player.payment = 0
-                player.handContent = None
+                player.invalidateHand()
         for box in self.wonBoxes:
             box.setVisible(False)
         self.draw.setChecked(False)
@@ -985,13 +987,8 @@ class ScoringDialog(QWidget):
                 self.nameLabels[idx].setBuddy(self.wonBoxes[idx])
                 for loop in range(10):
                     prevTotal = player.handTotal
-                    oldWinner = self.game.winner
-                    try:
-                        self.game.winner = player
-                        player.handContent = player.computeHand()
-                    finally:
-                        self.game.winner = oldWinner
-                    self.wonBoxes[idx].setVisible(player.handContent.won)
+                    handContent = player.computeHand(asWinner=True)
+                    self.wonBoxes[idx].setVisible(handContent.won)
                     if not self.wonBoxes[idx].isVisibleTo(self) and self.wonBoxes[idx].isChecked():
                         self.wonBoxes[idx].setChecked(False)
                         self.game.winner = None
@@ -1000,7 +997,6 @@ class ScoringDialog(QWidget):
                     player.refreshManualRules()
                 self.spValues[idx].setValue(player.handTotal)
             else:
-                player.handContent = player.computeHand()
                 if not self.spValues[idx].isEnabled():
                     self.spValues[idx].clear()
                     self.spValues[idx].setValue(0)
@@ -1023,7 +1019,7 @@ class ScoringDialog(QWidget):
         if self.game.winner and self.game.winner.handBoard:
             winnerTiles = self.game.winner.handBoard.tiles
             pairs = []
-            for meld in self.game.winner.computeHand().melds:
+            for meld in self.game.winner.hand.melds:
                 if len(meld) < 4:
                     pairs.extend(meld.pairs)
             for tile in winnerTiles:
@@ -1148,7 +1144,7 @@ class ScoringDialog(QWidget):
             if self.cbLastTile.count() == 0:
                 return
             lastTile = InternalParameters.field.computeLastTile()
-            winnerMelds = [m for m in self.game.winner.computeHand().melds if len(m) < 4 \
+            winnerMelds = [m for m in self.game.winner.hand.melds if len(m) < 4 \
                 and lastTile in m.pairs]
             assert len(winnerMelds)
             if len(winnerMelds) == 1:
