@@ -47,12 +47,13 @@
 #include <kselectaction.h>
 
 // KGame includes
-#include <kgamechat.h>
-#include <dialogs/kgamedialog.h>
-#include <dialogs/kgamedialogconfig.h>
-#include <dialogs/kgameconnectdialog.h>
-#include <dialogs/kgameerrordialog.h>
-#include <dialogs/kgamedebugdialog.h>
+#define USE_UNSTABLE_LIBKDEGAMESPRIVATE_API
+#include <libkdegamesprivate/kgame/kgamechat.h>
+#include <libkdegamesprivate/kgame/dialogs/kgamedialog.h>
+#include <libkdegamesprivate/kgame/dialogs/kgamedialogconfig.h>
+#include <libkdegamesprivate/kgame/dialogs/kgameconnectdialog.h>
+#include <libkdegamesprivate/kgame/dialogs/kgameerrordialog.h>
+#include <libkdegamesprivate/kgame/dialogs/kgamedebugdialog.h>
 
 // application specific includes
 #include "chatdlg.h"
@@ -201,11 +202,11 @@ KWin4App::KWin4App(QWidget *parent)
 KWin4App::~KWin4App()
 {
   kDebug() << "~KWin4App()";
-  if (mDoc) delete mDoc;
-  if (mView) delete mView;
-  if (mScene) delete mScene;
-  if (mTheme) delete mTheme;
-  if (mMyChatDlg) delete mMyChatDlg;
+  delete mDoc;
+  delete mView;
+  delete mScene;
+  delete mTheme;
+  delete mMyChatDlg;
   kDebug() << "~KWin4App()";
 }
 
@@ -327,16 +328,16 @@ void KWin4App::initGUI()
   KStandardGameAction::hint(this, SLOT(askForHint()), actionCollection());
   KStandardGameAction::quit(this, SLOT(close()), actionCollection());
   
-  action = actionCollection()->addAction("network_conf");
+  action = actionCollection()->addAction( QLatin1String( "network_conf" ));
   action->setText(i18n("&Network Configuration..."));
-  connect(action, SIGNAL(triggered(bool) ), SLOT(configureNetwork()));
+  connect(action, SIGNAL(triggered(bool)), SLOT(configureNetwork()));
 
-  action = actionCollection()->addAction("network_chat");
+  action = actionCollection()->addAction( QLatin1String( "network_chat" ));
   action->setText(i18n("Network Chat..."));
-  connect(action, SIGNAL(triggered(bool) ), SLOT(configureChat()));
+  connect(action, SIGNAL(triggered(bool)), SLOT(configureChat()));
 
-  action = actionCollection()->addAction("statistics");
-  action->setIcon(KIcon("view-statistics"));
+  action = actionCollection()->addAction( QLatin1String( "statistics" ));
+  action->setIcon(KIcon( QLatin1String( "view-statistics" )));
   action->setText(i18n("&Show Statistics"));
   connect(action, SIGNAL(triggered(bool)), SLOT(showStatistics()));
   action->setToolTip(i18n("Show statistics."));
@@ -351,8 +352,8 @@ void KWin4App::initGUI()
   QStringList themes(mThemeFiles.keys());
   themes.sort();
 
-  action = actionCollection()->addAction("theme", new KSelectAction(i18n("Theme"), this));
-  action->setIcon(KIcon("games-config-theme"));
+  action = actionCollection()->addAction( QLatin1String( "theme" ) , new KSelectAction(i18n("Theme" ), this));
+  action->setIcon(KIcon( QLatin1String( "games-config-theme" )));
   ((KSelectAction*)action)->setItems(themes);
   connect( action, SIGNAL(triggered(int)), SLOT(changeTheme(int)) );
   kDebug() << "Setting current theme item to" << mThemeIndexNo;
@@ -361,9 +362,9 @@ void KWin4App::initGUI()
   // Debug
   if (global_debug>0)
   {
-    action = actionCollection()->addAction("file_debug");
+    action = actionCollection()->addAction( QLatin1String( "file_debug" ));
     action->setText(i18n("Debug KGame"));
-    connect(action, SIGNAL(triggered(bool) ), SLOT(debugKGame()));
+    connect(action, SIGNAL(triggered(bool)), SLOT(debugKGame()));
   }
 }
 
@@ -397,12 +398,12 @@ void KWin4App::initStatusBar()
 void KWin4App::connectDocument()
 {
   // KGame signals
-  connect(mDoc,SIGNAL(signalGameOver(int, KPlayer*,KGame*)),
-         this,SLOT(slotGameOver(int, KPlayer*,KGame *)));
+  connect(mDoc,SIGNAL(signalGameOver(int,KPlayer*,KGame*)),
+         this,SLOT(slotGameOver(int,KPlayer*,KGame*)));
   connect(mDoc,SIGNAL(signalNextPlayer(int)),
          this,SLOT(moveDone(int)));
-  connect(mDoc,SIGNAL(signalClientLeftGame(int, int,KGame*)),
-         this,SLOT(networkBroken(int, int, KGame*)));
+  connect(mDoc,SIGNAL(signalClientLeftGame(int,int,KGame*)),
+         this,SLOT(networkBroken(int,int,KGame*)));
   connect(mDoc,SIGNAL(signalGameRun()),
          this,SLOT(gameRun()));
 }
@@ -851,8 +852,8 @@ void KWin4App::configureChat()
       mMyChatDlg->setPlayer(mDoc->getPlayer(Yellow));
     else
       mMyChatDlg->setPlayer(mDoc->getPlayer(Red));
-    connect(mDoc,SIGNAL(signalChatChanged(KWin4Player *)),
-            mMyChatDlg,SLOT(setPlayer(KWin4Player *)));
+    connect(mDoc,SIGNAL(signalChatChanged(KWin4Player*)),
+            mMyChatDlg,SLOT(setPlayer(KWin4Player*)));
   }
 
   if (mMyChatDlg->isHidden())
@@ -900,7 +901,7 @@ void KWin4App::configureSettings()
   QWidget* frame = new QWidget(dialog);
   ui.setupUi(frame);
   dialog->addPage(frame, i18n("General"), "games-config-options");
-  connect(dialog, SIGNAL(settingsChanged(const QString &)), mDoc, SLOT(loadSettings()));
+  connect(dialog, SIGNAL(settingsChanged(QString)), mDoc, SLOT(loadSettings()));
   dialog->show();
 }
 
