@@ -24,12 +24,11 @@
 #include "kgrcanvas.h"
 #include "kgrselector.h"
 
-#ifdef ENABLE_SOUND_SUPPORT
-    #ifdef KGOLDRUNNER_USE_OPENAL
+// KGoldrunner loads and plays .ogg files and requires OpenAL + SndFile > v0.21.
+// Fallback to Phonon by the KgSound library does not give good results.
+#include <libkdegames_capabilities.h>
+#ifdef KGAUDIO_BACKEND_OPENAL
     #include "kgrsounds.h"
-    #else
-    #include "kgrsoundbank.h"
-    #endif
 #endif
 
 #include "kgreditor.h"
@@ -95,6 +94,7 @@ KGrGame::KGrGame (KGrCanvas * theView,
         userDataDir   (theUserDir),
         level         (0),
         mainDemoName  ("demo"),
+        // mainDemoName  ("CM"), // IDW test.
         demoType      (DEMO),
         startupDemo   (false),
         programFreeze (false),
@@ -417,13 +417,13 @@ void KGrGame::setInitialTheme (const QString & themeFilepath)
 
 void KGrGame::initGame()
 {
-#ifndef ENABLE_SOUND_SUPPORT
-    KGrMessage::information (view, i18n ("No Sound"),
-        i18n ("Warning: This copy of KGoldrunner has no sound.\n"
-              "\n"
-              "This is because no development versions of the OpenAL and "
-              "SndFile libraries were present when it was compiled and built."),
-              "WarningNoSound");
+#ifndef KGAUDIO_BACKEND_OPENAL
+        KGrMessage::information (view, i18n ("No Sound"),
+            i18n ("Warning: This copy of KGoldrunner has no sound.\n"
+                  "\n"
+                  "This is because no development versions of the OpenAL and "
+                  "SndFile libraries were present when it was compiled and built."),
+                  "WarningNoSound");
 #endif
     kDebug() << "Entered, draw the initial graphics now ...";
 
@@ -469,19 +469,19 @@ void KGrGame::initGame()
                                                       "champion_speed"), true);
     timeScale = gameGroup.readEntry ("ActualSpeed", 10);
 
-#ifdef ENABLE_SOUND_SUPPORT
-    // Set up sounds, if required in config.
-    soundOn = gameGroup.readEntry ("Sound", false);
-    kDebug() << "Sound" << soundOn;
-    if (soundOn) {
-        loadSounds();
-        effects->setMuted (false);
-    }
-    emit setToggle ("options_sounds", soundOn);
+#ifdef KGAUDIO_BACKEND_OPENAL
+        // Set up sounds, if required in config.
+        soundOn = gameGroup.readEntry ("Sound", false);
+        kDebug() << "Sound" << soundOn;
+        if (soundOn) {
+            loadSounds();
+            effects->setMuted (false);
+        }
+        emit setToggle ("options_sounds", soundOn);
 
-    stepsOn = gameGroup.readEntry ("StepSounds", false);
-    kDebug() << "StepSounds" << stepsOn;
-    emit setToggle ("options_steps", stepsOn);
+        stepsOn = gameGroup.readEntry ("StepSounds", false);
+        kDebug() << "StepSounds" << stepsOn;
+        emit setToggle ("options_steps", stepsOn);
 #endif
 
     dbk1 << "Owner" << gameList.at (gameIndex)->owner
@@ -900,25 +900,25 @@ void KGrGame::incScore (const int n)
 
 void KGrGame::playSound (const int n, const bool onOff)
 {
-#ifdef ENABLE_SOUND_SUPPORT
-    if (! effects) {
-        return;			// Sound is off and not yet loaded.
-    }
-    static int fallToken = -1;
-    if (onOff) {
-	int token = -1;
-        if (stepsOn || ((n != StepSound) && (n != ClimbSound))) {
-            token = effects->play (fx [n]);
+#ifdef KGAUDIO_BACKEND_OPENAL
+        if (! effects) {
+            return;            // Sound is off and not yet loaded.
         }
-        if (n == FallSound) {
-            fallToken = token;
+        static int fallToken = -1;
+        if (onOff) {
+        int token = -1;
+            if (stepsOn || ((n != StepSound) && (n != ClimbSound))) {
+                token = effects->play (fx [n]);
+            }
+            if (n == FallSound) {
+                fallToken = token;
+            }
+        }
+        // Only the falling sound can get individually turned off.
+        else if ((n == FallSound) && (fallToken >= 0)) {
+        effects->stop (fallToken);
+        fallToken = -1;
         }
-    }
-    // Only the falling sound can get individually turned off.
-    else if ((n == FallSound) && (fallToken >= 0)) {
-	effects->stop (fallToken);
-	fallToken = -1;
-    }
 #endif
 }
 
@@ -926,7 +926,7 @@ void KGrGame::endLevel (const int result)
 {
     dbk << "Return to KGrGame, result:" << result;
 
-#ifdef ENABLE_SOUND_SUPPORT
+#ifdef KGAUDIO_BACKEND_OPENAL
     if (effects) {			// If sounds have been loaded, cut off
         effects->stopAllSounds();	// all sounds that are in progress.
     }
@@ -1159,7 +1159,8 @@ void KGrGame::toggleSoundsOnOff (const int action)
     else {
         stepsOn = soundOnOff;
     }
-#ifdef ENABLE_SOUND_SUPPORT
+
+#ifdef KGAUDIO_BACKEND_OPENAL
     if (action == PLAY_SOUNDS) {
         if (soundOn && (effects == 0)) {
             loadSounds();	// Sounds were not loaded when the game started.
@@ -1175,7 +1176,7 @@ void KGrGame::freeze (const bool userAction, const bool on_off)
     kDebug() << "PAUSE:" << type << on_off;
     kDebug() << "gameFrozen" << gameFrozen << "programFreeze" << programFreeze;
 
-#ifdef ENABLE_SOUND_SUPPORT
+#ifdef KGAUDIO_BACKEND_OPENAL
     if (on_off && effects) {		// If pausing and sounds are loaded, cut
         effects->stopAllSounds();	// off all sounds that are in progress.
     }
@@ -1230,8 +1231,21 @@ void KGrGame::showHint()
     // Put out a hint for this level.
     QString caption = i18n ("Hint");
 
-    if (levelHint.length() > 0)
-        myMessage (view, caption, levelHint);
+    if (levelHint.length() > 0) {
+	freeze (ProgramPause, true);
+	// TODO - IDW. Check if a solution exists BEFORE showing the extra button.
+	switch (KGrMessage::warning (view, caption, levelHint,
+			    i18n ("&OK"), i18n ("&Show A Solution"))) {
+	case 0:
+	    freeze (ProgramPause, false);	// No replay requested.
+	    break;
+	case 1:
+	    freeze (ProgramPause, false);	// Replay a solution.
+	    // This deletes current KGrLevelPlayer and play, but no life is lost.
+            runReplay (SOLVE, gameIndex, level);
+	    break;
+	}
+    }
     else
         myMessage (view, caption,
                         i18n ("Sorry, there is no hint for this level."));
@@ -2152,46 +2166,41 @@ bool KGrGame::loadRecording (const QString & dir, const QString & prefix,
 
 void KGrGame::loadSounds()
 {
-#ifdef ENABLE_SOUND_SUPPORT
-    const qreal volumes [NumSounds] = {0.6, 0.3, 0.3, 0.6, 0.6, 1.8, 1.0, 1.0, 1.0, 1.0};
-    #ifdef KGOLDRUNNER_USE_OPENAL
-    effects = new KGrSounds();
-    #else
-    effects = new KGrSoundBank (8);
-    #endif
-    effects->setParent (this);		// Delete at end of KGrGame.
+#ifdef KGAUDIO_BACKEND_OPENAL
+        const qreal volumes [NumSounds] = {0.6, 0.3, 0.3, 0.6, 0.6, 1.8, 1.0, 1.0, 1.0, 1.0};
+        effects = new KGrSounds();
+        effects->setParent (this);        // Delete at end of KGrGame.
 
-    fx[GoldSound]      = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/gold.wav"));
-    fx[StepSound]      = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/step.wav"));
-    fx[ClimbSound]     = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/climb.wav"));
-    fx[FallSound]      = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/falling.wav"));
-    fx[DigSound]       = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/dig.wav"));
-    fx[LadderSound]    = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/ladder.wav"));
-    fx[CompletedSound] = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/completed.wav"));
-    fx[DeathSound]     = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/death.wav"));
-    fx[GameOverSound]  = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/gameover.wav"));
-    fx[VictorySound]   = effects->loadSound (KStandardDirs::locate ("appdata",
-                         "themes/default/victory.wav"));
-    #ifdef KGOLDRUNNER_USE_OPENAL
-    // Gold and dig sounds are timed and are allowed to play for at least one
-    // second, so that rapid sequences of those sounds are heard as overlapping.
-    effects->setTimedSound (fx[GoldSound]);
-    effects->setTimedSound (fx[DigSound]);
+        fx[GoldSound]      = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/gold.ogg"));
+        fx[StepSound]      = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/step.wav"));
+        fx[ClimbSound]     = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/climb.wav"));
+        fx[FallSound]      = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/falling.ogg"));
+        fx[DigSound]       = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/dig.ogg"));
+        fx[LadderSound]    = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/ladder.ogg"));
+        fx[CompletedSound] = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/completed.ogg"));
+        fx[DeathSound]     = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/death.ogg"));
+        fx[GameOverSound]  = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/gameover.ogg"));
+        fx[VictorySound]   = effects->loadSound (KStandardDirs::locate ("appdata",
+                             "themes/default/victory.ogg"));
 
-    // Adjust the relative volumes of sounds to improve the overall balance.
-    for (int i = 0; i < NumSounds; i++) {
-        effects->setVolume (fx [i], volumes [i]);
-    }
-    #endif
+        // Gold and dig sounds are timed and are allowed to play for at least one
+        // second, so that rapid sequences of those sounds are heard as overlapping.
+        effects->setTimedSound (fx[GoldSound]);
+        effects->setTimedSound (fx[DigSound]);
+
+        // Adjust the relative volumes of sounds to improve the overall balance.
+        for (int i = 0; i < NumSounds; i++) {
+            effects->setVolume (fx [i], volumes [i]);
+        }
 #endif
 }
 
